module Shaun
  # The SHAUN pretty printer. It prints a SHAUN value so that it is easy to read
  class PrettyPrinter
    # Create a pretty printer with the tabulation string (2 spaces by default)
    def initialize(tabchar = '  ')
      @tab = tabchar
      @indent_level = 0
    end

    # Add a level of indentation
    def indent
      @indent_level = @indent_level + 1
    end

    # remove a level of indentation
    def dedent
      @indent_level = @indent_level - 1
    end

    # Get the indentation string
    def indent_string
      @tab * @indent_level
    end

    # Pretty print a number
    def print_number(sn_num)
      sn_num.to_s
    end

    # Pretty print a boolean
    def print_bool(b)
      b.to_s
    end

    # Pretty print a string
    def print_string(s)
      "\"#{s}\""
    end

    # Pretty print a null value
    def print_nil
      "null"
    end

    # Pretty print a list
    def print_list(ary)
      indent
      contents = ary.map { |e| print_list_element e }.join "\n"
      dedent
      "[\n" + contents + "\n" + indent_string + "]"
    end

    # Pretty print an object
    def print_object(obj)
      indent

      attributes = obj.instance_variables.map do |name|
        value = obj.instance_variable_get name
        attr_name = name.to_s.sub /@/, ''
        print_object_attribute attr_name, value
      end

      contents = attributes.join "\n"
      dedent
      "{\n" + contents + "\n" + indent_string + "}"
    end

    private
      def print_object_attribute(name, value)
        contents = "#{name}: #{value.pretty_print(self)}"
        indent_string + contents
      end

      def print_list_element(elem)
        indent_string + elem.pretty_print(self)
      end
  end
end
